// SmileyRacer.java
//
// ICS 21 / CSE 21 Summer 2010
// Lab Assignment 3
//
// Originally coded by Norm Jacobson, October 2006
// Minor modifications introduced by Alex Thornton, Summer 2009

import java.awt.Color;
import java.util.Random;


// A SmileyRacer is a special kind of SmileyFace that is capable of
// participating in a smiley race and tracking its own progress.
// When we say that SmileyRacer "extends SmileyFace", we're saying
// that SmileyRacer objects can do anything that SmileyFace objects
// can do, and can do some other things additionally.  The methods
// and fields in SmileyFace are automatically "inherited" into SmileyRacer,
// making them available to us without us having to copy them into the
// SmileyRacer class.
public class SmileyRacer extends SmileyFace
{
	// The name of this smiley, which is displayed on top of the smiley
	// in the graphics window, and also in the statistics area after the
	// race concludes.
	private String smileyName;
	
	// The color that should be used to show the name of the smiley in the
	// graphics window.
	private Color smileyNameColor;
	
	// A random number generator, to be used in choosing speeds at random.
	private Random generator;

	// The number of ticks that this smiley has been running.
	private int ticks;
	
	// The direction that the smiley is currently moving: RIGHT or LEFT.
	private int directionMoving;
	
	// The number of laps that have been completed by this smiley.
	private int lapsCompleted;
	
	// The number of pixels this smiley moves per tick (i.e., its present
	// speed).
	private int pixelsToMovePerTick;
	
	// The speed change that this smiley will make when each lap is
	// completed.  A steady (unchanging) speed would be indicated by
	// the value 0 in this field; a negative value would indicate
	// that the smiley will get slower after every lap; a positive
	// value means it would get faster.
	private int perLapSpeedChange;
	
	// The distance that the smiley should be moved after each lap, which
	// is part of what needs to be done to make the smiley appear to
	// reverse direction.
	private int distanceToMoveSmile;

	// The fastest possible speed that a smiley may have.
	private static final int MAX_MOVEMENT_PER_TICK = 10;
	
	// Value to store in directionMoving to indicate that this smiley is
	// moving to the right.
	private static final int RIGHT = 1;
	
	// Value to store in directionMoving to indicate that this smiley is
	// moving to the left.
	private static final int LEFT = -1;

	
	// This constructor initializes a SmileyRacer from an existing SmileyFace.
	// (Because SmileyRacers are also SmileyFaces, you can also use an
	// existing SmileyRacer.)  You also need to pass in its name and the
	// name's color.
	public SmileyRacer(SmileyFace existingFace, String name, Color nameColor)
	{
		// To build a racer, we first build the smiley face itself
		super(existingFace);

		// Initialize the smiley's name and name color.
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***

		// Translate the SmileyRacer so that its leftmost point is at the
		// left edge of the window.  This ensures that there's no cheating
		// -- everyone has to start at the left edge.
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***

		// Construct a random number generator.
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***

		// Initialize ticks and laps completed to zero.  Set the current
		// direction to RIGHT.
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***

		// When a racer hits a wall, it needs to face the other direction.
		// To do this, the mouth has to move to the other side of the face,
		// so that it's off-center on one side by the same amount that it's
		// off-center presently.  The distance that the smile should move
		// is twice the distance that the center of the smile is away from
		// the center of the face (i.e., how far off-center is it?).
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***

		// Choose the current speed randomly, so that it lies between 1 and
		// MAX_MOVEMENT_PER_TICK (inclusive).
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***
		
		// Choose the per-lap speed change randomly, so that it lies between
		// 0 and pixelsToMovePerTick / 2.  Choose its sign (i.e., positive or
		// negative) randomly, with a 50% chance of it being an increase and
		// a 50% chance of it being a decrease.  (Hint: Generate a 0 or 1
		// randomly; if a 0 is generated, make perLapSpeedChange negative.)
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***
	}


	// finishedRace() returns true if the SmileyRacer has finished the race,
	// false if not.
	public boolean finishedRace()
	{
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***
	}


	// raceForOneTick() moves the racer forward the distance that it moves
	// for one tick.  Also, it increases the number of ticks that the racer
	// has been in the race.
	//
	// A loop is a handy way to solve this problem: loop over the number of
	// pixels that the racer should move, moving one pixel forward and
	// checking if adjustments are needed (e.g., reverse direction when
	// edge is hit) each iteration.
	public void raceForOneTick()
	{
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***
	}


	// moveForwardOnePixel() moves the racer forward one pixel, if it has
	// not already finished the race.  If the wall has been hit, be sure to
	// make the appropriate adjustments (i.e., reversing direction, changing
	// speed).
	//
	// Remember when adjusting the speed that, post-adjustment, the speed is
	// not permitted to be outside of the range 1..MAX_MOVEMENT_PER_TICK.
	private void moveForwardOnePixel()
	{
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***
	}


	// reverseProfile() reverses the appearance of the racer, by swapping
	// the colors of its left and right eyes and moving its smile to the
	// opposite side of its face.
	private void reverseProfile()
	{
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***
	}


	// hitSomething() returns true if the racer has hit either wall.
	private boolean hitSomething()
	{
		// *** REPLACE THIS COMMENT WITH YOUR CODE ***
	}


	// hitLeftWall() returns true if the racer has hit the left wall, false
	// otherwise.
	private boolean hitLeftWall()
	{
		return
			(getFace().getLeftmostX() <=
			 SmileyDisplay.LEFT_EDGE && directionMoving == LEFT);
	}

	
	// hitRightWall() returns true if the racer has hit the right wall,
	// false otherwise.
	private boolean hitRightWall()
	{
		return (getFace().getLeftmostX() + getFace().getXLength() >=
					SmileyDisplay.RIGHT_EDGE && directionMoving == RIGHT);
	}


	// Accessors

	public int getTicks()
	{
		return ticks;
	}

	public String getSmileyName()
	{
		return smileyName;
	}

	public Color getSmileyNameColor()
	{
		return smileyNameColor;
	}
	
	public int getLapsCompleted()
	{
		return lapsCompleted;
	}
}
